@@ -8,7 +8,7 @@ from .tree import DWARFTreeModel, has_code_location, cu_sort_key
 from .scriptdlg import ScriptDlg
 from .ui import setup_ui
 
-version = (1, 11)
+version = (1, 22)
 
 # TODO:
 # On MacOS, start without a main window, instead show the Open dialog
@@ -104,6 +104,7 @@ class TheWindow(QMainWindow):
                     cu._i = i
             di._locparser = None # Created on first use
 
+            self.dwarfinfo = di
             self.tree_model = DWARFTreeModel(di, self.prefix, self.sortcus, self.sortdies)
             self.the_tree.setModel(self.tree_model)
             self.the_tree.selectionModel().currentChanged.connect(self.on_tree_selection)
@@ -335,10 +336,14 @@ class TheWindow(QMainWindow):
             self.on_findnext()
 
     def on_findip(self):
-        r = QInputDialog.getText(self, "Find code address", "Offset (hex), assuming the module is loaded at its preferred address:")
+        start_address = hex(self.dwarfinfo._start_address) if not self.dwarfinfo._start_address is None else "its preferred address"
+        r = QInputDialog.getText(self, "Find code address", "Code address (hex), assuming the module is loaded at %s:" % start_address)
         if r[1] and r[0]:
             try:
-                ip = int(r[0], 16)
+                ip = r[0]
+                if r[0].startswith("0x"):
+                    ip = ip[2:]
+                ip = int(ip, 16)
                 self.findcondition = lambda die: ip_in_range(die, ip)
                 self.findcucondition = lambda cu: ip_in_range(cu.get_top_DIE(), ip)
                 self.findnext_menuitem.setEnabled(True)
@@ -364,7 +369,7 @@ class TheWindow(QMainWindow):
     ##########################################################################
 
     def on_about(self):
-        QMessageBox(QMessageBox.Icon.Information, "About...", "DWARF Explorer v." + '.'.join(str(v) for v in version) + "\n\nSeva Alekseyev, 2020\nsevaa@sprynet.com\n\ngithub.com/sevaa/dwex",
+        QMessageBox(QMessageBox.Icon.Information, "About...", "DWARF Explorer v." + '.'.join(str(v) for v in version) + "\n\nSeva Alekseyev, 2020-2021\nsevaa@sprynet.com\n\ngithub.com/sevaa/dwex",
             QMessageBox.Ok, self).show()
 
     def on_updatecheck(self):
@@ -390,6 +395,7 @@ class TheWindow(QMainWindow):
 
     def on_exit(self):
         self.destroy()
+        QApplication.quit()
 
     # Checkmark toggling is handled by the framework
     def on_view_prefix(self, checked):
@@ -431,7 +437,8 @@ class TheWindow(QMainWindow):
         if self.tree_model:
             sel = self.the_tree.currentIndex()
             sel = self.tree_model.set_sortcus(checked, sel) # This will reload the tree
-            self.the_tree.setCurrentIndex(sel)
+            if sel:
+                self.the_tree.setCurrentIndex(sel)
 
     def on_sortdies(self, checked):
         self.sortdies = checked
@@ -446,7 +453,8 @@ class TheWindow(QMainWindow):
             # Navigation stack - empty
             self.navhistory = []
             self.navpos = -1
-            self.the_tree.setCurrentIndex(sel)
+            if sel:
+                self.the_tree.setCurrentIndex(sel)
 
     def on_highlight_code(self):
         self.highlightcode_menuitem.setChecked(True)
@@ -516,10 +524,13 @@ class TheWindow(QMainWindow):
         QApplication.restoreOverrideCursor()
 
 def on_exception(exctype, exc, tb):
-    from .crash import report_crash
-    report_crash(exc, tb, version)
-    sys.excepthook = on_exception.prev_exchook
-    sys.exit(1)
+    if isinstance(exc, Exception):
+        from .crash import report_crash
+        report_crash(exc, tb, version)
+        sys.excepthook = on_exception.prev_exchook
+        sys.exit(1)
+    elif on_exception.prev_exchook:
+        on_exception.prev_exchook(exctype, exc, tb)
 
 class TheApp(QApplication):
     def __init__(self):
@@ -539,7 +550,7 @@ class TheApp(QApplication):
         self.exec_()
 
 def main():     
-    if sys.settrace is None: # Lame way to detect a debugger
+    if not sys.gettrace(): # Lame way to detect a debugger
         on_exception.prev_exchook = sys.excepthook
         sys.excepthook = on_exception
 
